@@ -59,21 +59,23 @@ mocked = []
 def lookup_by_name(name, nsdicts):
     """
     Look up an object by name from a sequence of namespace dictionaries.
-    Returns a tuple of (nsdict, object, attributes); nsdict is the
-    dictionary the name was found in, object is the name of the base object
-    the name is bound to, and the attributes list is the chain of attributes
-    of the object that complete the name.
+    Returns a tuple of (nsdict, obj_name, attrs); nsdict is the namespace
+    dictionary the name was found in, obj_name is the name of the base object
+    the name is bound to, and the attrs list is the chain of attributes
+    of the object that completes the name.
 
         >>> import os
-        >>> nsdict, name, attributes = lookup_by_name("os.path.isdir", 
+        >>> nsdict, obj_name, attrs = lookup_by_name("os.path.isdir", 
         ...     (locals(),))
-        >>> name, attributes
+        >>> obj_name, attrs
         ('os', ['path', 'isdir'])
-        >>> nsdict, name, attributes = lookup_by_name("os.monkey", (locals(),))
+        >>> getattr(getattr(nsdict[obj_name], attrs[0]), attrs[1])
+        <function isdir at ...>
+        >>> lookup_by_name("os.monkey", (locals(),))
         Traceback (most recent call last):
           ...
         NameError: name 'os.monkey' is not defined
-            
+
     """
     for nsdict in nsdicts:
         attrs = name.split(".")
@@ -165,6 +167,22 @@ def mock(name, nsdicts=None, mock_obj=None, **kw):
         >>> Test.sm()
         'sm'
 
+    Test mocking a proxy object::
+
+        >>> class Proxy(object):
+        ...     def __init__(self, obj):
+        ...         self._obj = obj
+        ...     def __getattr__(self, name):
+        ...         return getattr(self._obj, name)
+        >>> import os
+        >>> os = Proxy(os)
+        >>> os.path.isfile
+        <function isfile at ...>
+        >>> mock('os.path.isfile')
+        >>> os.path.isfile
+        <Mock ... os.path.isfile>
+        >>> restore()
+
     """
     if nsdicts is None:
         stack = inspect.stack()
@@ -196,8 +214,14 @@ def mock(name, nsdicts=None, mock_obj=None, **kw):
         nsdict[obj_name] = mock_obj
     else:
         for attr in attrs[:-1]:
-            tmp = tmp.__dict__[attr]
-        original = tmp.__dict__[attrs[-1]]
+            try:
+                tmp = tmp.__dict__[attr]
+            except KeyError:
+                tmp = getattr(tmp, attr)
+        try:
+            original = tmp.__dict__[attrs[-1]]
+        except KeyError:
+            original = getattr(tmp, attrs[-1])
         setattr(tmp, attrs[-1], mock_obj)
 
     mocked.append((original, nsdict, obj_name, attrs))
@@ -217,7 +241,10 @@ def restore():
         else:
             tmp = nsdict[name]
             for attr in attrs[:-1]:
-                tmp = tmp.__dict__[attr]
+                try:
+                    tmp = tmp.__dict__[attr]
+                except KeyError:
+                    tmp = getattr(tmp, attr)
             setattr(tmp, attrs[-1], original)
 
 def assert_same_trace(tracker, want):
@@ -504,7 +531,7 @@ class Mock(object):
             'mock_returns_func',
             'mock_returns_iter',
             'mock_tracker',
-            'show_attrs',
+            'mock_show_attrs',
             )):
             if attr == 'mock_returns_iter' and value is not None:
                 value = iter(value)
@@ -541,6 +568,10 @@ __test__ = {
     ...     pass
     ... else:
     ...     raise AssertionError('m() should have raised ValueError')
+    >>> m.mock_tracker = Printer(sys.stdout)
+    >>> m.mock_show_attrs = True
+    >>> m.a = 2
+    Set mock_obj.a = 2
     """,
 
     "mock" :
